package engine;

import gui.views.inside.ControlPanel;
import gui.views.inside.DisplayPanel;
import gui.windows.MainWindow;

import java.awt.Point;

import javax.swing.JLabel;
import javax.swing.JScrollPane;

import main.ElevatorSimulator;
import properties.ProgramSettings;
import simulator.controllers.$controlling_controllertype;
import simulator.model.Elevator;

/**
 * Klasse mit eigener RunnableInstanz
 * 
 */
public class ElevatorEngine implements Runnable {

	private MainWindow mw;

	private $controlling_controllertype myController;
	private Elevator myElevator;
	private JLabel myElevatorImage;
	private ControlPanel myControls;
	private DisplayPanel myDisplays;

	#if($controlling_accelerated == true)
	private SpeedController speedController;
	#end
	
	/**
	 * Konstruktor der Klasse ElevatorThread
	 * 
	 * @param iControllerIndex
	 */
	public ElevatorEngine($controlling_controllertype controller) {
		mw = MainWindow.getInstance();
		
		#if($elevators > 1)
		myController = ElevatorSimulator.getInstance().getController(controller.getIndex());
		myElevatorImage = mw.getElevatorPanel(controller.getIndex()).getElevatorImage();
		myControls = mw.getControlPanel(controller.getIndex());
		myDisplays = mw.getDisplayPanel(controller.getIndex());
		#else
		myController = ElevatorSimulator.getInstance().getController();
		myElevatorImage = mw.getElevatorPanel().getElevatorImage();
		myControls = mw.getControlPanel();
		myDisplays = mw.getDisplayPanel();
		#end
		
		myElevator = controller.getElevator();
		
		#if($controlling_accelerated == true)
		speedController = new SpeedController();
		#end
	}

	@Override
	/**
	 * Ueberladene run-Methode, welche mittels Schleife die Position des Grafikelements verndert
	 */
	public void run() {
		// synchronized (mw.getlElevatorPanels().get(iControllerIndex)) {
		#if($innerview_doorbutton == true)
		myControls.getCloseButton().setEnabled(false);
		myControls.getOpenButton().setEnabled(false);
		#end

		while (myController.getCurrentTarget() != null) {

			if (myController.isUpdateNecessary()) {
				myController.updateTarget();
			}

			if (!myController.getDeletionQueue().contains(
					myController.getCurrentTarget().getFloor())) {
				
				int floorHeight;
				floorHeight = ProgramSettings.FLOORS_HEIGHT;

				
				// Bewegungsanimation des Fahrstuhls
				for (int i = 0; i < floorHeight; i++) {

					// /////////////////////////////////////
					myElevatorImage.setLocation((int) myElevatorImage.getLocation().getX(),
							(int) myElevatorImage.getLocation().getY() - myElevator.getDirection());

					#if($outerview_autoscroll)
						scrollWithElevator(myElevator);
					#end

						
						
						try {
							#if($controlling_accelerated == true)
							speedController.adaptSpeed(i, myElevator, myControls.getButtons());
							#else
							Thread.sleep(ProgramSettings.ELEVATORS_SPEED.getValue());
							#end
						} catch (InterruptedException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
						

				}
			}

			myElevator.setbCloseDoorsNow(false);
			myElevator.setbKeepDoorsOpen(false);

			myDisplays.updateValues(myElevator, myController.getCurrentTarget());

			boolean targetFound = myController.simulate();
			
			#if($display_direction == true)
			myDisplays.changeImageDirection(myElevator.getDirection());
			#end

			if (targetFound) {
				#if($innerview_doorbutton == true)
				/*
				 * Tueren oeffnen und schliessen Buttons koennen nur betaetigt
				 * werden, wenn der Fahrstuhl sein Ziel erreicht hat bzw. steht.
				 */
				myControls.getCloseButton().setEnabled(true);
				myControls.getOpenButton().setEnabled(true);
				#end

				mw.animateDoors(myController.getIndex(), myElevator.getCurrentFloor());
				myController.doAfterAnimate();

				#if($innerview_doorbutton == true)
				myControls.getCloseButton().setEnabled(false);
				myControls.getOpenButton().setEnabled(false);
				#end

			}

		}
		#if($innerview_doorbutton == true)
		myControls.getCloseButton().setEnabled(true);
		myControls.getOpenButton().setEnabled(true);
		#end
	}

	
	#if($outerview_autoscroll)
	/**
	 * Diese Methode laesst die Scrollbar mit dem Elevator scrollen, wenn er aus
	 * dem Bild laeuft. wenn der Elevator aus dem Viewpoint rausfaehrt, dannsoll
	 * die scrollbar mitscrollen, also der viewpoint sollsich verschieben.
	 * 
	 * wenn die y-koordinate von dem Elevator kleiner ist alsDifferenz aus
	 * Gesamthoehe des Panels und der hoehe des Viewport, dann soll der
	 * viewpoint verschoben werden.
	 * 
	 * @param elevator
	 *            - der Elevator fuer die Location
	 * 
	 */
	public void scrollWithElevator(Elevator elevator) {

		/*
		 * wenn der Elevator aus dem Viewpoint rausfaehrt, dannsoll die
		 * scrollbar mitscrollen, also der viewpoint soll sich verschieben.
		 */
		
		Point pImage = new Point(myElevatorImage.getLocation());
		JScrollPane jscroll = mw.getScrollPane(#if($elevators > 1) myController.getIndex() #end);

		if (elevator.getDirection() == 1) {
			if (pImage.getY() < (jscroll.getViewport().getViewPosition().getY())) {
				jscroll.getViewport().setViewPosition(
						new Point((int) 1, (int) jscroll.getViewport().getViewPosition().getY()
								- (elevator.getDirection() * jscroll.getHeight())));
			}
		} else if (elevator.getDirection() == -1) {
			if (pImage.getY() > (jscroll.getViewport().getViewPosition().getY() + 500)) {
				jscroll.getViewport().setViewPosition(
						new Point((int) 1, (int) jscroll.getViewport().getViewPosition().getY()
								- (elevator.getDirection() * jscroll.getHeight())));

			}
		}

	}
	#end
}