package gui.views.inside;

#if($display_currfloor_chain == true)
import gui.views.inside.displays.FloorChainDisplay;
#end

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;

import simulator.model.Elevator;
import simulator.model.Target;

/**
 * 
 * @author Rob
 * Diese Klasse reprsentiert ein Panel, welches alle Anzeige-Elemente
 * des Fahrstuhls enthlt
 */
public class DisplayPanel extends JPanel {
	
	private static final long serialVersionUID = -7053155399480364159L;
	
	//VARIATIONSPUNKT
	//Feature: aktuelle Etage (Zahl)
	#if($display_currfloor_number == true)
	private JLabel lblCurrentFloor;
	#end
	
	//VARIATIONSPUNKT
	//Feature: aktuelle Etage (Kette)
	#if($display_currfloor_chain == true)
	private FloorChainDisplay chainDisplay;
	#end
	
	#if($display_target == true)
	private JLabel lblTargetFloor;
	#end
	
	#if($display_openingside == true)
	private JLabel lblDoorOpen;
	#end
	
	#if($display_direction == true)
	private JLabel lblDirection;
	#end
	
	private JLabel lblController;
	
	private String sControllerName;
	
	private final int PANEL_WIDTH = 500;
	private final int PANEL_HEIGHT = 320;
	

	
	#if($display_target == true)
	/**
	 * @return the lblTargetFloor
	 */
	public JLabel getLblTargetFloor() {
		return lblTargetFloor;
	}
	#end

	#if($display_openingside == true)
	/**
	 * @return the lblDoorOpen
	 */
	public JLabel getLblDoorOpen() {
		return lblDoorOpen;
	}
	#end

	#if($display_direction == true)
	/**
	 * @return the lblDirection
	 */
	public JLabel getLblDirection() {
		return lblDirection;
	}
	#end

	/**
	 * @return the lblController
	 */
	public JLabel getLblController() {
		return lblController;
	}
	

	
	public DisplayPanel(String sControllerName) {
		super();
		this.sControllerName = sControllerName;
		init();
	}
	
	public void init() {
		
		this.setPreferredSize(new Dimension(PANEL_WIDTH, PANEL_HEIGHT));
		this.setLayout(new BorderLayout());
		// das Panel fur space, darein kommt jpStatus

		JPanel jpStatus = new JPanel();
		jpStatus.setLayout(new GridLayout(7, 2));

		// font fuer die Anzeigen
		Font font = new Font("Arial", Font.PLAIN, 13);
		
		// Controller-Typ
		lblController = new JLabel();
		lblController.setFont(font);
		
		
		lblController.setText(sControllerName);
		
		jpStatus.add(new JLabel("Controller:"));
		jpStatus.add(lblController);
		
		#if($display_currfloor_number == true)
		// aktuelle Etage (Zahl)
		lblCurrentFloor = new JLabel("0");
		lblCurrentFloor.setFont(font);
		jpStatus.add(new JLabel("Aktuelle Etage: "));
		jpStatus.add(lblCurrentFloor);
		#end
		
		#if($display_currfloor_chain == true)
		// aktuelle Etage (Kette)
		chainDisplay = new FloorChainDisplay();
		this.add(chainDisplay, BorderLayout.NORTH);
		#end
		
		#if($display_target == true)
		// Zieletage
		lblTargetFloor = new JLabel("keine");
		lblTargetFloor.setFont(font);
		jpStatus.add(new JLabel("Zieletage:"));
		jpStatus.add(lblTargetFloor);
		#end
		
		#if($display_openingside == true)
		// Tr-Zustand
		lblDoorOpen = new JLabel();
		lblDoorOpen.setFont(font);
		jpStatus.add(new JLabel("Tr:"));
		jpStatus.add(lblDoorOpen);
		#end

		#if($display_direction == true)
		// Fahrtrichtung
		lblDirection = new JLabel();
		Icon icArrowZ = new ImageIcon(getClass().getResource("/stop.gif"));
		lblDirection.setIcon(icArrowZ);
		jpStatus.add(new JLabel("Richtung:"));
		jpStatus.add(lblDirection);
		#end
		
		jpStatus.setPreferredSize(new Dimension(400, 300));
		this.add(jpStatus, BorderLayout.CENTER);

		// Panels fuer abstaende
		this.add(new JPanel(), BorderLayout.EAST);
		
	}
	
	/**
	 * Anzeige der Statuswerte des Fahrstuhls aktualisieren
	 */
	public void updateValues(Elevator elevator, Target currentTarget) {
		
		int iCurrentFloor = elevator.getCurrentFloor();
		int iDirection = elevator.getDirection();
		
		#if($display_currfloor_number == true)
		lblCurrentFloor.setText(Integer.toString(iCurrentFloor + iDirection));
		#end
		
		#if($display_currfloor_chain == true)
		chainDisplay.setValue(iCurrentFloor + iDirection);
		#end
		
		#if($display_target == true)
		if (currentTarget != null) {
			lblTargetFloor.setText(Integer.toString(currentTarget.getFloor()));
		}
		#end
	}
	
	
	#if($display_direction == true)
	/**
	 * Diese Methode wechselt das Bild der Richtungsanzeige, je nachedem, ob der
	 * Fahrstuhl steht, nach oben oder nach unten faehrt.
	 */
	public void changeImageDirection(int iDirection) {

		switch(iDirection) {
			case 1:
				lblDirection.setIcon(new ImageIcon(getClass().getResource(
				"/up.gif")));
				break;
			case -1:
				lblDirection.setIcon(new ImageIcon(getClass().getResource(
				"/down.gif")));
				break;
			default:
				lblDirection.setIcon(new ImageIcon(getClass().getResource(
				"/stop.gif")));
				break;
		}
		
	}
	#end
	
}
